function view_mendeldata (experiment)
% A function for the program mendelHMM to visualize the phenotype data from Mendel's paper.
% The same datasets are also used by the function resample_mendeldata,
% but here we include all the samples in the 1 gene case, and
% we only study the F2 and F3 generations.
% Ref.: Stern, C and. Sherwood, E.R.(1966), The Origin of genetics: a
% Mendel source book. pp 10-15 (experiment 1-7), pp 17-19 (emperiment 8)
% and pp 20-21 (emperiment 9).

% From the mendelHMM-toolbox by Steinar Thorvaldsen, 2004.
% http://www.math.uit.no/bi/hmm/

% Input:
% experiment: the number of the dataset to plot

figure('Name',['mendelHMM: View of dataset ']), clf 
if experiment >=1 & experiment <=7 % the 1 gene case:
    % The dominants/recisives in generation F1, F2 and F3 with all 7 genes:
    data= [253 258 929 1181 580 858 1064 5474 6022 705 882 428 651 787 372 353 64 71 60 67 72
             0   0   0    0   0   0    0 1850 2001 224 299 152 207 277 193 166 36 29 40 33 28];
    % (Only about 10% of the dominants in F2 were tested future in F3.
    % In experiment 1 and 2 the numbers i F2 and F3 is high because 
    % about 30 seeds grow from each plant.)
    for i=8:21 
        N=data(1,i)+data(2,i);
        pr(1,i-7)=data(1,i)/N; % extracting prob. from F2 and F3 in one vector
        SD(1,i-7)=sqrt(pr(i-7)*(1-pr(i-7))/N); % Standard deviation of estimated parameters p
    end
    plot(pr,'gs','MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',5)
    title ('Mendels 7 experiments with 1 gene at a time, and 90% confidence intervals')
    xlabel ('1-7: F2 generation.   8-14: F3 generation. ');
    ylabel ('Frequency of the dominant phenotype');
    axis([0 16 0.5 1]);
    hold on;
    
    % We also want to plot the 90% confidence intervals in the same diagram.
    % The probability distribution of the phenotypes are binomial(N,p).
    % The estimator of p has a gaussian distribution, with the standard
    % deviaton given abow.
    z=1.645; % statistical parameter giving the 90% confidence interval
    error=SD*z; % The 90% confidence inteval
    errorbar (1:14, pr(1:14), error,'+'); % plots confidence intervals

    % To plot the parameter level p in the model:
    p=3/4; % In F2 generation
    y(1:7)=p;
    plot (1:7, y, ':'); % plot parameter in F2 generation
    p=2/3; % The same in F3 generation
    y(1:7)=p;
    plot (8:14, y, ':'); % plot parameter in F3 generation
    hold off;
    
end %if

if experiment ==8 % the F1, F2 and F3 for 2 genes:
    data = [529 301 209
              0  96 120
              0 102 127
              0  30  73];
    % extracting fraction from F2 and F3 in one vector:
    for i=1:4 fraction(1,i)=data(i,2); end
    for i=1:4 fraction(1,i+4)=data(i,3); end
    fraction = (fraction/529)*16; % to plot fraction in units of 16
    plot(fraction,'gs','MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',5)
    title ('Mendelian experiment with 2 genes (4 phenotypes) ')
    xlabel ('1-4: F2 generation.       5-8: F3 generation. ');
    ylabel ('Frequency (in fraction of 16)');
    axis([0 9 0 10]);
    
    hold on;
    x=1:1:4; % plotting limiting lines
    y(1:4)=4;
    plot (x, y, ':');
    x=5:1:8;
    plot (x, y, ':');
    hold off;
end % if

if experiment ==9 % the F1, F2 and F3 for 3 genes
    data = [639 269 153
              0  88  92
              0  86  92
              0  30  56
              0  98 100
              0  34  60
              0  27  55
              0   7  31];
    % extracting fraction from F2 and F3 in one vector:
    for i=1:8 fraction(1,i)=data(i,2); end
    for i=1:8 fraction(1,i+8)=data(i,3); end
    fraction = (fraction/639)*64; % to plot fraction in units of 64
    plot(fraction,'gs','MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',5)
    title ('Mendelian experiment with 3 genes (8 phenotypes)')
    xlabel ('1-8: F2 generation.         9-16: F3 generation. ');
    ylabel ('Frequency (in fraction of 64)');
    axis([0 17 0 28]);
    hold on;
    x=1:1:8; % plotting limiting lines
    y(1:8)=8;
    plot (x, y, ':');
    x=9:1:16;
    plot (x, y, ':');
    hold off;
end % if
% hold off;
